/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package wf_info

import (
	"encoding/json"
	"fmt"
	"gitlab.nrao.edu/ssa/gocapo/capo/config"
	"io"
	"net/http"
)

type WfInfo struct {
	WfId       int64  `json:"workflow_request_id"`
	WfName     string `json:"workflow_name"`
	ResultsDir string `json:"results_dir"`
	Argument   `json:"argument"`
}

type Argument struct {
	CachePath string `json:"cache_path"`
}

//checkError
/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

//GetSECIWorkflowInfo
/**
 * Get Workflow info for SECI Ingestion
 *
 * @param wf_req_id A string of a workflow request id
 * @param capo_profile A string with the path to the CAPO property file used to
 * get the workflow service url
 * @return a WfInfo struct
 **/
func GetSECIWorkflowInfo(wfReqId string, capoProfile string, path string) WfInfo {
	properties, err := config.InitConfig(capoProfile, path)
	workflowUrl := properties.SettingsForPrefix("edu.nrao.workspaces.WorkflowSettings").GetString("serviceUrl")

	workflowEndpoint := fmt.Sprintf("%s/workflows/ingest_seci/requests/%s", workflowUrl, wfReqId)

	resp, err := http.Get(workflowEndpoint)
	checkError(err)

	if resp.StatusCode != http.StatusOK {
		panic(fmt.Sprintf("Error making GET request to URL %s, got status code %d", workflowEndpoint, resp.StatusCode))
	}

	defer func(Body io.ReadCloser) {
		err := Body.Close()
		checkError(err)
	}(resp.Body)

	wfInfo := new(WfInfo)
	err = json.NewDecoder(resp.Body).Decode(wfInfo)
	checkError(err)

	if wfInfo.WfName != "ingest_seci" {
		panic(fmt.Sprintf("The provided workflow request id %s is not an ingest_seci workflow. Cannot provide status.", wfReqId))
	}

	return *wfInfo
}
